@@ -1,15 +1,16 @@
-/* Copyright (c) 2007, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2007, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,11 +35,11 @@
 #include "my_psi_config.h"
 #include "my_sqlcommand.h"
 #include "my_sys.h"
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
-#include "mysql/components/services/mysql_mutex_bits.h"
-#include "mysql/components/services/psi_mutex_bits.h"
 #include "mysql/mysql_lex_string.h"
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_mutex.h"
@@ -69,7 +70,7 @@ class Audit_error_handler : public Internal_error_handler {
   /**
     @brief Blocked copy constructor (private).
   */
-  Audit_error_handler(const Audit_error_handler &obj MY_ATTRIBUTE((unused)))
+  Audit_error_handler(const Audit_error_handler &obj [[maybe_unused]])
       : m_thd(nullptr),
         m_warning_message(nullptr),
         m_error_reported(false),
@@ -309,6 +310,7 @@ inline const CHARSET_INFO *thd_get_audit_query(THD *thd,
   } else {
     query->str = thd->query().str;
     query->length = thd->query().length;
+    DBUG_PRINT("print_query", ("%.*s\n", (int)query->length, query->str));
     return thd->charset();
   }
 }
@@ -346,9 +348,8 @@ class Ignore_event_error_handler : public Audit_error_handler {
     @param sqlstate  The SQL state of the underlying error. NULL if none
     @param msg       The text of the underlying error. NULL if none
   */
-  void print_warning(const char *warn_msg MY_ATTRIBUTE((unused)),
-                     uint sql_errno, const char *sqlstate,
-                     const char *msg) override {
+  void print_warning(const char *warn_msg [[maybe_unused]], uint sql_errno,
+                     const char *sqlstate, const char *msg) override {
     LogErr(WARNING_LEVEL, ER_AUDIT_CANT_ABORT_EVENT, m_event_name, sql_errno,
            sqlstate ? sqlstate : "<NO_STATE>", msg ? msg : "<NO_MESSAGE>");
   }
@@ -384,7 +385,13 @@ int mysql_audit_notify(THD *thd, mysql_event_general_subclass_t subclass,
   event.general_host = sctx->host();
   event.general_external_user = sctx->external_user();
   event.general_rows = thd->get_stmt_da()->current_row_for_condition();
-  event.general_sql_command = sql_statement_names[thd->lex->sql_command];
+
+  if (msg != nullptr && thd->lex->sql_command == SQLCOM_END &&
+      thd->get_command() != COM_QUERY) {
+    event.general_sql_command = {STRING_WITH_LEN("")};
+  } else {
+    event.general_sql_command = sql_statement_names[thd->lex->sql_command];
+  }
 
   event.general_charset = const_cast<CHARSET_INFO *>(
       thd_get_audit_query(thd, &event.general_query));
@@ -492,7 +499,7 @@ int mysql_audit_notify(THD *thd, mysql_event_parse_subclass_t subclass,
 
   @retval true - generate event, otherwise not.
 */
-inline bool generate_table_access_event(THD *thd, TABLE_LIST *table) {
+inline bool generate_table_access_event(THD *thd, Table_ref *table) {
   /* Discard views or derived tables. */
   if (table->is_view_or_derived()) return false;
 
@@ -544,7 +551,7 @@ inline static void set_table_access_subclass(
 */
 static int mysql_audit_notify(THD *thd,
                               mysql_event_table_access_subclass_t subclass,
-                              const char *subclass_name, TABLE_LIST *table) {
+                              const char *subclass_name, Table_ref *table) {
   LEX_CSTRING str;
   mysql_event_table_access event;
 
@@ -571,7 +578,7 @@ static int mysql_audit_notify(THD *thd,
                                     subclass_name, &event);
 }
 
-int mysql_audit_table_access_notify(THD *thd, TABLE_LIST *table) {
+int mysql_audit_table_access_notify(THD *thd, Table_ref *table) {
   mysql_event_table_access_subclass_t subclass;
   const char *subclass_name;
   int ret;
@@ -812,9 +819,8 @@ class Ignore_command_start_error_handler : public Audit_error_handler {
     @param sqlstate  The SQL state of the underlying error. NULL if none
     @param msg       The text of the underlying error. NULL if none
   */
-  void print_warning(const char *warn_msg MY_ATTRIBUTE((unused)),
-                     uint sql_errno, const char *sqlstate,
-                     const char *msg) override {
+  void print_warning(const char *warn_msg [[maybe_unused]], uint sql_errno,
+                     const char *sqlstate, const char *msg) override {
     LogErr(WARNING_LEVEL, ER_AUDIT_CANT_ABORT_COMMAND, m_command_text,
            sql_errno, sqlstate ? sqlstate : "<NO_STATE>",
            msg ? msg : "<NO_MESSAGE>");
@@ -1155,12 +1161,15 @@ void mysql_audit_release(THD *thd) {
     data->release_thd(thd);
   }
 
+  /* Move audit plugins array from THD to a tmp variable in order to avoid calls
+     to them via THD after the plugin is unlocked (#bug34594035) */
+  Plugin_array audit_class_plugins(std::move(thd->audit_class_plugins));
+
   /* Now we actually unlock the plugins */
-  plugin_unlock_list(nullptr, thd->audit_class_plugins.begin(),
-                     thd->audit_class_plugins.size());
+  plugin_unlock_list(nullptr, audit_class_plugins.begin(),
+                     audit_class_plugins.size());
 
   /* Reset the state of thread values */
-  thd->audit_class_plugins.clear();
   thd->audit_class_mask.clear();
   thd->audit_class_mask.resize(MYSQL_AUDIT_CLASS_MASK_SIZE);
 }
@@ -1296,7 +1305,7 @@ static bool calc_class_mask(THD *, plugin_ref plugin, void *arg) {
 int finalize_audit_plugin(st_plugin_int *plugin) {
   unsigned long event_class_mask[MYSQL_AUDIT_CLASS_MASK_SIZE];
 
-  if (plugin->plugin->deinit && plugin->plugin->deinit(nullptr)) {
+  if (plugin->plugin->deinit && plugin->plugin->deinit(plugin)) {
     DBUG_PRINT("warning", ("Plugin '%s' deinit function returned error.",
                            plugin->name.str));
     DBUG_EXECUTE("finalize_audit_plugin", return 1;);
@@ -1420,7 +1429,7 @@ static int event_class_dispatch_error(THD *thd, mysql_event_class_t event_class,
 }
 
 /**  There's at least one active audit plugin tracking a specified class */
-bool is_audit_plugin_class_active(THD *thd MY_ATTRIBUTE((unused)),
+bool is_audit_plugin_class_active(THD *thd [[maybe_unused]],
                                   unsigned long event_class) {
   return mysql_global_audit_mask[event_class] != 0;
 }
