@@ -994,7 +994,7 @@ static const byte *trx_undo_read_blob_update(const byte *undo_ptr,
 static byte *trx_undo_report_blob_update(page_t *undo_page, dict_index_t *index,
                                          byte *undo_ptr, const byte *field,
                                          ulint flen, const upd_t *update,
-                                         upd_field_t *fld, mtr_t *mtr) {
+                                         const upd_field_t *fld, mtr_t *mtr) {
   DBUG_TRACE;
 
   /* Access the LOB reference object. */
@@ -1316,7 +1316,7 @@ static ulint trx_undo_page_report_modify(
     in new table */
     if (dict_index_is_online_ddl(index) && index->table->n_v_cols > 0) {
       for (i = 0; i < upd_get_n_fields(update); i++) {
-        upd_field_t *fld = upd_get_nth_field(update, i);
+        const upd_field_t *fld = upd_get_nth_field(update, i);
         ulint pos = fld->field_no;
 
         /* These columns must not have an index
@@ -1331,7 +1331,7 @@ static ulint trx_undo_page_report_modify(
     ptr += mach_write_compressed(ptr, n_updated);
 
     for (i = 0; i < upd_get_n_fields(update); i++) {
-      upd_field_t *fld = upd_get_nth_field(update, i);
+      const upd_field_t *fld = upd_get_nth_field(update, i);
 
       bool is_virtual = upd_fld_is_virtual_col(fld);
       bool is_multi_val = upd_fld_is_multi_value_col(fld);
